#pragma semicolon 1

#include <sourcemod>
#include <sdktools>
#include <sdkhooks>

#define PLUGIN_NAME 	"Radar Config"
#define PLUGIN_VERSION 	"1.1.0"

new Handle:g_hRadarConfig = INVALID_HANDLE;

new g_iFlashDuration = -1;
new g_iFlashAlpha = -1;
new bool:g_bFlashHooked;

new g_iPlayerSpotted = -1;
new g_iBombSpotted = -1;
new g_iPlayerManager = -1;
new bool:g_bRadarHooked;

public Plugin:myinfo =
{
	name = PLUGIN_NAME,
	author = "GoD-Tony",
	description = "Hide or Show all players on the radar",
	version = PLUGIN_VERSION,
	url = "http://www.sourcemod.net/"
};

public OnPluginStart()
{
	/* Find offsets used for the radar and flashbangs. */
	if ((g_iPlayerSpotted = FindSendPropOffs("CCSPlayerResource", "m_bPlayerSpotted")) == -1)
		SetFailState("Failed to find CCSPlayerResource::m_bPlayerSpotted offset");

	if ((g_iBombSpotted = FindSendPropOffs("CCSPlayerResource", "m_bBombSpotted")) == -1)
		SetFailState("Failed to find CCSPlayerResource::m_bBombSpotted offset");
	
	if ((g_iFlashDuration = FindSendPropOffs("CCSPlayer", "m_flFlashDuration")) == -1)
		SetFailState("Failed to find find CCSPlayer::m_flFlashDuration offset");
	
	if ((g_iFlashAlpha = FindSendPropOffs("CCSPlayer", "m_flFlashMaxAlpha")) == -1)
		SetFailState("Failed to find CCSPlayer::m_flFlashMaxAlpha offset");
	
	/* Now we can continue. */
	CreateConVar("sm_radarconfig_version", PLUGIN_VERSION, PLUGIN_NAME, FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY|FCVAR_DONTRECORD);
	g_hRadarConfig = CreateConVar("sm_radarconfig", "0", "Determines radar functionality. (0 = Default behaviour, 1 = Disable radar, 2 = Show all players)", FCVAR_NONE, true, 0.0, true, 2.0);
	
	OnRadarModeChange(g_hRadarConfig, "", "");
	HookConVarChange(g_hRadarConfig, OnRadarModeChange);
}

public OnMapStart()
{
	g_iPlayerManager = FindEntityByClassname(0, "cs_player_manager");
	
	if (g_bRadarHooked)
		SDKHook(g_iPlayerManager, SDKHook_PostThink, OnEntityThink);
}

public OnClientPutInServer(client)
{
	if (g_bRadarHooked)
		SDKHook(client, SDKHook_PreThink, OnEntityThink);
}

public Event_PlayerSpawn(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));
	
	if (client && GetClientTeam(client) > 1)
		Client_HideRadar(client);
}

public Event_PlayerBlind(Handle:event, const String:name[], bool:dontBroadcast)
{
	new userid = GetEventInt(event, "userid");
	new client = GetClientOfUserId(userid);
	new Float:duration = GetEntDataFloat(client, g_iFlashDuration);
	
	if (client && GetClientTeam(client) > 1)
		CreateTimer(duration, Timer_FlashEnd, userid, TIMER_FLAG_NO_MAPCHANGE);
}

public Action:Timer_FlashEnd(Handle:timer, any:userid)
{
	new client = GetClientOfUserId(userid);
	
	if (client && GetClientTeam(client) > 1)
		Client_HideRadar(client);
		
	return Plugin_Stop;
}

public OnEntityThink(entity)
{
	for (new i = 0; i <= 65; i++)
		SetEntData(g_iPlayerManager, g_iPlayerSpotted + i, true, 4, true);
	
	SetEntData(g_iPlayerManager, g_iBombSpotted, true, 4, true);
}

public OnRadarModeChange(Handle:convar, const String:oldValue[], const String:newValue[])
{
	new iRadarMode = GetConVarInt(convar);

	switch (iRadarMode)
	{
		case 0: // Default behaviour
		{
			if (g_bRadarHooked)
				Unhook_Radar();
			
			if (g_bFlashHooked)
				Unhook_Flash();
		}
		
		case 1: // Disable radar for all
		{
			if (g_bRadarHooked)
				Unhook_Radar();
			
			if (!g_bFlashHooked)
				Hook_Flash();
		}
		
		case 2: // Show all players on radar
		{
			if (!g_bRadarHooked)
				Hook_Radar();
			
			if (g_bFlashHooked)
				Unhook_Flash();
		}
	}
}

Hook_Radar()
{
	g_bRadarHooked = true;
	
	SDKHook(g_iPlayerManager, SDKHook_PostThink, OnEntityThink);
	
	for (new i = 1; i <= MaxClients; i++)
		if (IsClientConnected(i) && IsClientInGame(i))
			SDKHook(i, SDKHook_PreThink, OnEntityThink);
}

Unhook_Radar()
{
	g_bRadarHooked = false;
	
	SDKUnhook(g_iPlayerManager, SDKHook_PostThink, OnEntityThink);
	
	for (new i = 1; i <= MaxClients; i++)
		if (IsClientConnected(i) && IsClientInGame(i))
			SDKUnhook(i, SDKHook_PreThink, OnEntityThink);
}

Hook_Flash()
{
	g_bFlashHooked = true;
	HookEvent("player_spawn", Event_PlayerSpawn, EventHookMode_Post);
	HookEvent("player_blind", Event_PlayerBlind, EventHookMode_Post);
	
	for (new i = 1; i <= MaxClients; i++)
		if (IsClientConnected(i) && IsClientInGame(i) && GetClientTeam(i) > 1)
			Client_HideRadar(i);
}

Unhook_Flash()
{
	g_bFlashHooked = false;
	UnhookEvent("player_spawn", Event_PlayerSpawn, EventHookMode_Post);
	UnhookEvent("player_blind", Event_PlayerBlind, EventHookMode_Post);
	
	for (new i = 1; i <= MaxClients; i++)
		if (IsClientConnected(i) && IsClientInGame(i) && GetClientTeam(i) > 1)
			Client_ShowRadar(i);
}

Client_HideRadar(client)
{
	SetEntDataFloat(client, g_iFlashDuration, 3600.0, true);
	SetEntDataFloat(client, g_iFlashAlpha, 0.5, true);
}

Client_ShowRadar(client)
{
	SetEntDataFloat(client, g_iFlashDuration, 0.5, true);
	SetEntDataFloat(client, g_iFlashAlpha, 0.5, true);
}